//////////////////////////////////////////////////////////////////////////
// Gp32misc: GP32 Keyboard and cpu API emulation library
// Copyright (C) 2008-2009, hardyx
// This library can be used freely under the GNU GPL license.
//////////////////////////////////////////////////////////////////////////

#include "gp32emu.h"
#include <stdlib.h>
#include <SDL/SDL.h>

#ifdef GP2X
  // acceso al hardware de mem
  #include <sys/mman.h>
  #include <fcntl.h>
  #define OFF_GPIOH    (0x0837)
  #define PWR_TFT_BIT  (2)
  #define SYS_CLK_FREQ 7372800
#endif

// permitir cambiar velocidad
#undef USE_CPUSPEED
// usar el botn Y en vez de start
#undef USESTART_FY

// GP2X button mapping
enum MAP_KEY
{
	VK_UP = 0,      VK_LEFT = 2,
  VK_DOWN = 4,    VK_RIGHT = 6,
	VK_START = 8,	  VK_SELECT = 9,
	VK_FL = 11,	    VK_FR = 10,
  VK_FA = 12,	    VK_FB = 13,
	VK_FY = 14,	    VK_FX = 15,
	VK_VOLUP = 16,	VK_VOLDWN = 17
};

//////////////////////////////////////////////////////////
// funciones y variables privadas

// intercambiar las teclas A y B como GP32
static bool m_swapab = false;

extern bool m_sdlInit;
extern SDL_Joystick *m_joystick;

//////////////////////////////////////////////////////////
// funciones de teclado y varias

// no hace nada
void GpKeyInit(void)
{
}

// intercambia las teclas a y b
void GpSwapABKeys(short swap)
{
	m_swapab = (swap==1);
}

// da la tecla pulsada, no bloqueante
int GpKeyGet(void)
{
	if(!m_sdlInit)
		return GPC_VK_NONE;

	SDL_Event _event;
	if(SDL_PollEvent(&_event)) { // no bloquea
		if(_event.type == SDL_QUIT)
			exit(0); // exit app
	}

	int res = GPC_VK_NONE;
#ifdef WIN32
  Uint8 *keys = SDL_GetKeyState(NULL);
	if(keys != NULL) {
		if(keys[SDLK_LEFT])
			res = GPC_VK_LEFT;
		else if(keys[SDLK_RIGHT])
			res = GPC_VK_RIGHT;
		else if(keys[SDLK_UP])
			res = GPC_VK_UP;
		else if(keys[SDLK_DOWN])
			res = GPC_VK_DOWN;
		else if(keys[SDLK_l])
			res = GPC_VK_FL;
		else if(keys[SDLK_r])
			res = GPC_VK_FR;
		else if(keys[SDLK_a]) // a=b
			res = m_swapab? GPC_VK_FB: GPC_VK_FA;
		else if(keys[SDLK_b]) // b=a
			res = m_swapab? GPC_VK_FA: GPC_VK_FB;
		else if(keys[SDLK_s])
			res = GPC_VK_START;
		else if(keys[SDLK_RETURN])
			res = GPC_VK_SELECT;
	}
#else
	// botones de la gp2x
	if(m_joystick != NULL) {
		SDL_JoystickUpdate();
		SDL_Joystick *joy = m_joystick;
		if(SDL_JoystickGetButton(joy,VK_LEFT))
			res = GPC_VK_LEFT;
		else if(SDL_JoystickGetButton(joy,VK_RIGHT))
			res = GPC_VK_RIGHT;
		else if(SDL_JoystickGetButton(joy,VK_UP))
			res = GPC_VK_UP;
		else if(SDL_JoystickGetButton(joy,VK_DOWN))
			res = GPC_VK_DOWN;
		else if(SDL_JoystickGetButton(joy,VK_FL))
			res = GPC_VK_FL;
		else if(SDL_JoystickGetButton(joy,VK_FR))
			res = GPC_VK_FR;
		else if(SDL_JoystickGetButton(joy,VK_FA)) // a=b
			res =  m_swapab? GPC_VK_FB: GPC_VK_FA;
		else if(SDL_JoystickGetButton(joy,VK_FB)) // b=a
			res =  m_swapab? GPC_VK_FA: GPC_VK_FB;
#ifdef USESTART_FY
		else if(SDL_JoystickGetButton(joy,VK_FY)) // y=start
			res = GPC_VK_START;
#else
		else if(SDL_JoystickGetButton(joy,VK_START))
			res = GPC_VK_START;
#endif
		else if(SDL_JoystickGetButton(joy,VK_SELECT))
			res = GPC_VK_SELECT;
	}
#endif // gp2x
	return res;
}

unsigned int GpTickCountGet(void)
{
	return SDL_GetTicks();
}

// cambiar la velocidad de la cpu
void GpClockSpeed(int cpu_mhz)
{
#if defined(GP2X) && defined(USE_CPUSPEED)
  unsigned int MHZ = cpu_mhz;
  int memfd = open("/dev/mem",O_RDWR);
  unsigned long *memregs32 = (unsigned long*)mmap(0, 0x10000, PROT_READ|PROT_WRITE,
    MAP_SHARED, memfd, 0xc0000000);
  unsigned short *memregs16 = (unsigned short*)memregs32;
  unsigned int v;
  unsigned int mdiv,pdiv=3,scale=0;
  MHZ*=1000000;
  mdiv=(MHZ*pdiv)/SYS_CLK_FREQ;

  mdiv=((mdiv-8)<<8) & 0xff00;
  pdiv=((pdiv-2)<<2) & 0xfc;
  scale&=3;
  v = mdiv | pdiv | scale;

  unsigned int l = memregs32[0x808>>2];
  memregs32[0x808>>2] = 0xFF8FFFE7;
  memregs16[0x910>>1]=v;
  while(memregs16[0x0902>>1] & 1);
  memregs32[0x808>>2] = l;
#endif
}
